"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _debugLevels = _interopRequireDefault(require("@resolve-js/debug-levels"));

var _lambda = require("resolve-cloud-common/lambda");

var _apiGatewayHandler = _interopRequireDefault(require("./api-gateway-handler"));

var _deployServiceEventHandler = _interopRequireDefault(require("./deploy-service-event-handler"));

var _schedulerEventHandler = _interopRequireDefault(require("./scheduler-event-handler"));

var _initScheduler = _interopRequireDefault(require("./init-scheduler"));

var _initMonitoring = _interopRequireDefault(require("./init-monitoring"));

var _metrics = require("./metrics");

var _initResolve = _interopRequireDefault(require("../common/init-resolve"));

var _disposeResolve = _interopRequireDefault(require("../common/dispose-resolve"));

var _websocketEventHandler = _interopRequireDefault(require("./websocket-event-handler"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const log = (0, _debugLevels.default)('resolve:runtime:cloud-entry');
const GRACEFUL_WORKER_SHUTDOWN_TIME = 30 * 1000;

const getVacantTimeInMillis = lambdaContext => lambdaContext.getRemainingTimeInMillis() - GRACEFUL_WORKER_SHUTDOWN_TIME;

const EVENT_SUBSCRIBER_DIRECT = 'EventSubscriberDirect';
let coldStart = true;

const initSubscriber = (resolve, lambdaContext) => {
  resolve.eventSubscriberDestination = lambdaContext.invokedFunctionArn;
  resolve.subscriptionsCredentials = {
    applicationLambdaArn: lambdaContext.invokedFunctionArn
  };

  resolve.invokeEventSubscriberAsync = async (eventSubscriber, method, parameters) => {
    await (0, _lambda.invokeFunction)({
      FunctionName: lambdaContext.invokedFunctionArn,
      InvocationType: 'Event',
      Region: process.env.AWS_REGION,
      Payload: {
        resolveSource: EVENT_SUBSCRIBER_DIRECT,
        method,
        payload: {
          eventSubscriber,
          ...parameters
        }
      }
    });
  };
};

const lambdaWorker = async (resolveBase, lambdaEvent, lambdaContext) => {
  log.debug('executing application lambda');
  log.verbose('incoming event', JSON.stringify(lambdaEvent, null, 2));
  lambdaContext.callbackWaitsForEmptyEventLoop = false;
  (0, _initMonitoring.default)(resolveBase);
  const resolve = Object.create(resolveBase);
  resolve.getVacantTimeInMillis = getVacantTimeInMillis.bind(null, lambdaContext);
  const lambdaRemainingTimeStart = lambdaContext.getRemainingTimeInMillis();

  try {
    if (lambdaEvent.resolveSource === 'DeployService') {
      initSubscriber(resolveBase, lambdaContext);
      (0, _initScheduler.default)(resolve);
      log.debug('initializing reSolve framework');
      await (0, _initResolve.default)(resolve);
      log.debug('reSolve framework initialized');
      log.debug('identified event source: deployment service');
      const executorResult = await (0, _deployServiceEventHandler.default)(lambdaEvent, resolve);
      log.verbose(`executorResult: ${JSON.stringify(executorResult)}`);
      return executorResult;
    } else if (lambdaEvent.resolveSource === EVENT_SUBSCRIBER_DIRECT) {
      initSubscriber(resolveBase, lambdaContext);
      (0, _initScheduler.default)(resolve);
      log.debug('initializing reSolve framework');
      await (0, _initResolve.default)(resolve);
      log.debug('reSolve framework initialized');
      log.debug('identified event source: event-subscriber-direct');
      const {
        method,
        payload
      } = lambdaEvent;
      const actualPayload = method === 'build' ? { ...payload,
        coldStart
      } : payload;
      const executorResult = await resolve.eventSubscriber[method](actualPayload);
      log.verbose(`executorResult: ${JSON.stringify(executorResult)}`);
      return executorResult;
    } else if (lambdaEvent.resolveSource === 'Scheduler') {
      initSubscriber(resolveBase, lambdaContext);
      (0, _initScheduler.default)(resolve);
      log.debug('initializing reSolve framework');
      await (0, _initResolve.default)(resolve);
      log.debug('reSolve framework initialized');
      log.debug('identified event source: cloud scheduler');
      const executorResult = await (0, _schedulerEventHandler.default)(lambdaEvent, resolve);
      log.verbose(`executorResult: ${JSON.stringify(executorResult)}`);
      return executorResult;
    } else if (lambdaEvent.resolveSource === 'Websocket') {
      initSubscriber(resolveBase, lambdaContext);
      (0, _initScheduler.default)(resolve);
      log.debug('initializing reSolve framework');
      await (0, _initResolve.default)(resolve);
      log.debug('reSolve framework initialized');
      log.debug('identified event source: websocket');
      const executorResult = await (0, _websocketEventHandler.default)(lambdaEvent, resolve);
      log.verbose(`executorResult: ${JSON.stringify(executorResult)}`);
      return executorResult;
    } else if (lambdaEvent.headers != null && lambdaEvent.httpMethod != null) {
      initSubscriber(resolveBase, lambdaContext);
      (0, _initScheduler.default)(resolve);
      log.debug('initializing reSolve framework');
      await (0, _initResolve.default)(resolve);
      log.debug('reSolve framework initialized');
      log.debug('identified event source: API gateway');
      log.verbose(JSON.stringify(lambdaEvent.httpMethod, null, 2), JSON.stringify(lambdaEvent.headers, null, 2));
      const executorResult = await (0, _apiGatewayHandler.default)(lambdaEvent, lambdaContext, resolve);
      log.verbose(`executorResult: ${JSON.stringify(executorResult)}`);
      return executorResult;
    } else {
      throw new Error(`abnormal lambda execution on event ${JSON.stringify(lambdaEvent)}`);
    }
  } catch (error) {
    log.error('top-level event handler execution error!');
    resolve.monitoring.group({
      Part: 'Internal'
    }).error(error);

    if (error instanceof Error) {
      log.error('error', error.message);
      log.error('error', error.stack);
    } else {
      log.error(JSON.stringify(error));
    }

    throw error;
  } finally {
    await (0, _disposeResolve.default)(resolve);

    if (process.env.RESOLVE_PERFORMANCE_MONITORING) {
      await (0, _metrics.putDurationMetrics)(lambdaEvent, lambdaContext, coldStart, lambdaRemainingTimeStart);
    }

    coldStart = false;
    log.debug('reSolve framework was disposed. publishing metrics');
    await resolve.monitoring.publish();
    log.debug(`metrics published`);
  }
};

var _default = lambdaWorker;
exports.default = _default;
//# sourceMappingURL=lambda-worker.js.map